import TransformUtil from '@/utils/teansformUtil.js'
import * as Cesium from 'cesium'
const {cartesianToCartographic, cartographicToCartesian} = TransformUtil

/**
* 采集高度
* @param {*} viewer
* @param {*} cartesians：笛卡尔坐标组成的数组
* @param {*} type：'terrain' 获取地形高度，'model'获取模型高度
* @returns 采集结果：笛卡尔坐标组成的数组
*/
function getHeightByType(viewer,cartesians, type) {
   if (type === 'model') {
       return new Promise(async resolve => {
           try {
               let promise = viewer.scene.clampToHeightMostDetailed(cartesians)
               promise.then(updatedCartesians => resolve(updatedCartesians))
           } catch (e) {
               resolve(false)
           }
       })
   } else if (type = 'terrain') {
       // 首先将cartesians转为positions
       let positions = []
       cartesians.forEach(cartesian => {
           positions.push(cartesianToCartographic(cartesian))
       })
       let terrain = viewer.terrainProvider
       return new Promise(async resolve => {
           try {
               // 当前场景中没有使用地形
               if (!terrain) resolve(false)
               const promise = Cesium.sampleTerrainMostDetailed(terrain, positions);
               promise.then(updatedPositions => {
                //    let resultCartesians = []
                //    updatedPositions.forEach(position => {
                //        if (position) {
                //            // 采集成功，转为笛卡尔坐标
                //            resultCartesians.push(cartographicToCartesian(position))
                //        } else {
                //            // postion为undefined时，说明该位置采集失败
                //            // resultCartesians.push(position)
                //        }
                //    })
                //    resolve(resultCartesians)
                    resolve(updatedPositions.map(item=>item.height))
                 })
           } catch (e) {
               resolve(false)
           }
       })
   }
}
const cartesianToLnglat = (cartesian)=>{
    //  //笛卡尔空间坐标转WGS84弧度.
    //  const ellipsoid84 = Cesium.Ellipsoid.wgs84;
    //  const cartographicPosition = ellipsoid84.cartesianToCartographic(cartesian);
    const cartographicPosition = Cesium.Cartographic.fromCartesian(cartesian);
    return [
      Cesium.Math.toDegrees(cartographicPosition.longitude),
      Cesium.Math.toDegrees(cartographicPosition.latitude),
    ]
  }
// 防抖函数
function debounce(func, delay) {
    let timer = null;
    return function () {
        const context = this;
        const args = arguments;
        clearTimeout(timer);
        timer = setTimeout(() => {
            func.apply(context, args);
        }, delay);
    };
}
export {
    getHeightByType,
    cartesianToLnglat,
    debounce
}