import { feature } from '@turf/turf';
import * as Cesium from 'cesium'
import * as turf from "@turf/turf";
import { getHeightByType } from './common.js';
import { ElMessage } from 'element-plus';
import PrimitiveCluster from "@/utils/primitiveCluster.js";
import GCJ02TilingScheme from "@/utils/modules/tiling-scheme/GCJ02TilingScheme.js";
import AMapImageryProvider from "@/utils/modules/provider/AMapImageryProvider.js";
import OffsetUrlTemplateImageryProvider from "@/utils/OffsetUrlTemplateImageryProvider.js";
const dataLoading = {
    addWMTSLayer: function (viewer, layerUrl, layerName, epsgCode, extent, layerId, show) {
        let matrixIds = [];
        for (let i = 0; i < 20; i++) {
            matrixIds.push(epsgCode + ":" + i)
        }

        //1.新建ImageryProvider
        let wmtsImageryProvider = new Cesium.WebMapTileServiceImageryProvider({
            url: layerUrl, //服务地址，如：'http://localhost:8080/geoserver/gwc/service/wmts'
            layer: layerName, //图层名称，如：'tiger:tiger_roads'
            style: "",
            format: "image/png",
            tileMatrixSetID: epsgCode,
            tileMatrixLabels: matrixIds,
            rectangle: new Cesium.Rectangle(
                Cesium.Math.toRadians(extent[0]),
                Cesium.Math.toRadians(extent[1]),
                Cesium.Math.toRadians(extent[2]),
                Cesium.Math.toRadians(extent[3])
            ),
            // 切图策略采用地理坐标的方案，4326是没有进行投影的地理坐标系
            tilingScheme: new Cesium.GeographicTilingScheme({
                //此处很重要，很重要。。。
                numberOfLevelZeroTilesX: 2,
                numberOfLevelZeroTilesY: 1,
            }),
        });

        //2.新建imageryLayer
        let imageryLayer = new Cesium.ImageryLayer(wmtsImageryProvider, {
            // show: visible,//是否可见
            // alpha: opacity//透明度
        });
        // 添加自定义属性，id标识
        if (layerId) {
            imageryLayer.id = layerId
        }
        imageryLayer.show = show
        //3.将imageryLayer加入到viewer中
        viewer.imageryLayers.add(imageryLayer);

    },
    addGridLayer: async function (viewer, url, gridServiceName, gridDataCallBack, show) {
        fetch(url).then(res => res.json()).then(async result => {
            console.log("result", result)
            // 方式1.使用图元的方式加载
            const tmpPrimitives = new Cesium.PrimitiveCollection();
            const labelPrimitives = new Cesium.LabelCollection({ "name": gridServiceName + "_label" });
            tmpPrimitives.name = "分幅网格"
            labelPrimitives.name = "分幅网格"
            let gridData = [];
            let polylineApperance = new Cesium.PolylineMaterialAppearance({
                material: Cesium.Material.fromType('Color', {
                    color: new Cesium.Color(0, 0, 1.0, 1.0)
                })
            })
            result.features.forEach(feature => {
                const polylinePositions = Cesium.Cartesian3.fromDegreesArrayHeights(feature.geometry.coordinates[0].flat(2))
                const instance = new Cesium.GeometryInstance({
                    geometry: new Cesium.GroundPolylineGeometry({
                        positions: polylinePositions,
                        width: 2.0,
                        // loop: true,
                    }),
                    id: JSON.stringify(feature.properties)
                });
                tmpPrimitives.add(new Cesium.GroundPolylinePrimitive({
                    geometryInstances: instance,
                    appearance: polylineApperance,
                }));


                // 图层树中的网格数据
                gridData.push({
                    label: feature.properties.Text,
                    id: feature.properties.Text,
                    type: "3dTilesLayer",
                    layerConfig: {}
                })

                // 标签
                var polyCenter = Cesium.BoundingSphere.fromPoints(polylinePositions).center; //获取面的中心点
                polyCenter = Cesium.Ellipsoid.WGS84.scaleToGeodeticSurface(polyCenter);
                polyCenter.z += 200
                labelPrimitives.add({
                    position: polyCenter,
                    text: feature.properties.Text,
                    font: '24px sans-serif',
                    fillColor: Cesium.Color.WHITE,
                    // outlineColor : Cesium.Color.BLACK,
                    // outlineWidth : 1.0,
                    showBackground: true,
                    backgroundColor: new Cesium.Color(0.165, 0.165, 0.165, 0.8),
                    backgroundPadding: new Cesium.Cartesian2(7, 5),
                    pixelOffset: Cesium.Cartesian2.ZERO,
                    eyeOffset: Cesium.Cartesian3.ZERO,
                    horizontalOrigin: Cesium.HorizontalOrigin.CENTER,
                    verticalOrigin: Cesium.VerticalOrigin.TOP,
                    // translucencyByDistance:new Cesium.NearFarScalar(10, 1.0, 8000, 1.0),
                    scale: 1.0,
                    scaleByDistance: new Cesium.NearFarScalar(10, 1, 8000, 0.5),
                    // pixelOffsetScaleByDistance :  new Cesium.NearFarScalar(10, 1.0, 8000, 0.5),
                    heightReference: Cesium.HeightReference.NONE,
                    distanceDisplayCondition: undefined
                })
            })



            gridDataCallBack && gridDataCallBack(gridData);

            viewer.scene.primitives.add(tmpPrimitives);
            viewer.scene.primitives.add(labelPrimitives);

            //     // // 方式2.使用datasource的方式，entity
            //     // const dataSource = Cesium.GeoJsonDataSource.load(result, {
            //     //     fill: Cesium.Color.BLUE.withAlpha(0.2), // 设置面的填充颜色
            //     //     stroke: Cesium.Color.RED, // 设置边界的颜色
            //     //     strokeWidth: 3 ,// 设置边界的宽度
            //     //     clampToGround:true
            //     // });

            //     // dataSource.then(async function(dataSource) {
            //     //     dataSource.name = gridServiceName
            //     //     viewer.dataSources.add(dataSource);
            //     //     // viewer.zoomTo(dataSource);
            //     //     dataSource.show = show;
            //     //     let gridData = [];
            //     //     // 获取实体数组
            //     //     var neighborhoodEntities = dataSource.entities.values;
            //     //     for (var i = 0; i < neighborhoodEntities.length; i++) {
            //     //         var entity = neighborhoodEntities[i];

            //     //         if (Cesium.defined(entity.polygon)) {
            //     //             // Cesium 将 GeoJson 的属性数据存储在实体的properties 属性中
            //     //             entity.name = entity.properties.Text;
            //     //             gridData.push({
            //     //                 label: entity.properties.Text._value,
            //     //                 id: entity.properties.Text._value,
            //     //                 type: "3dTilesLayer",
            //     //                 layerConfig:{}
            //     //             })
            //     //             // 使用随机颜色样式作为填充材料.
            //     //             entity.polygon.material = Cesium.Color.fromRandom({
            //     //                 red : 0.1,
            //     //                 maximumGreen : 0.5,
            //     //                 minimumBlue : 0.5,
            //     //                 alpha : 0.01
            //     //             });

            //     //             // Tells the polygon to color the terrain.
            //     //             //ClassificationType.CESIUM_3D_TILE will color the 3D tileset, and ClassificationType.BOTH will color both the 3d tiles and terrain (BOTH is the default)
            //     //             entity.polygon.classificationType = Cesium.ClassificationType.BOTH;
            //     //             entity.polygon.outline= true;
            //     //             entity.polygon.outlineColor=Cesium.Color.BLUE;  // 设置轮廓颜色为蓝色
            //     //             entity.polygon.outlineWidth= 2
            //     //             entity.polygon.height = 0
            //     //             entity.polygon.perPositionHeight = false;
            //     //             entity.polygon.heightReference = Cesium.HeightReference.CLAMP_TO_GROUND

            //     //             // 获取面的位置坐标信息
            //     //             var polyPositions = entity.polygon.hierarchy.getValue(Cesium.JulianDate.now()).positions;
            //     //             var polyCenter = Cesium.BoundingSphere.fromPoints(polyPositions).center; //获取面的中心点
            //     //             polyCenter = Cesium.Ellipsoid.WGS84.scaleToGeodeticSurface(polyCenter);
            //     //             // const height = await getHeightByType(viewer, [polyCenter], 'terrain');
            //     //             // polyCenter.z += height?(height[0]+10):10;
            //     //             polyCenter.z += 200
            //     //             // polyCenter.y += 660 
            //     //             entity.position = polyCenter;
            //     //             // Generate labels
            //     //             entity.label = {
            //     //                 text : entity.name,
            //     //                 showBackground : true,//指定标签背景的可见性。
            //     //                 scale : 0.6,//原图像的比例
            //     //                 horizontalOrigin : Cesium.HorizontalOrigin.CENTER,//水平对齐方式
            //     //                 verticalOrigin : Cesium.VerticalOrigin.BOTTOM,//垂直对齐方式
            //     //                 distanceDisplayCondition : new Cesium.DistanceDisplayCondition(10.0, 80000.0),//设置于 Camera 距离的显示范围
            //     //                 disableDepthTestDistance : 100.0,
            //     //                 scaleByDistance: new Cesium.NearFarScalar(
            //     //                     1000.0, // 近距距离（单位：米）
            //     //                     0.8,    // 近距缩放值
            //     //                     5000.0, // 远距距离（单位：米）
            //     //                     0.8    // 远距缩放值
            //     //                 ),
            //     //             };
            //     //         }
            //     //     }
            //     //     gridDataCallBack(gridData)
            //     // });

            //     // 方式3.使用primitiveCluster的方式
        })
        /* 贴地线 */
        // let polyline = new Cesium.GeometryInstance({
        //     geometry: new Cesium.GroundPolylineGeometry({
        //         positions: Cesium.Cartesian3.fromDegreesArray([
        //             108.0, 31.0, 100.0, 36.0, 105.0, 39.0
        //         ]),
        //         width: 2.0
        //     })
        // })

        // let polylineApperance = new Cesium.PolylineMaterialAppearance({
        //     material: Cesium.Material.fromType('Color', {
        //         color: new Cesium.Color(1.0, 1.0, 0.0, 1.0)
        //     })
        // })

        // let addGroundPolylineGeometry = new Cesium.GroundPolylinePrimitive({
        //     geometryInstances: polyline,
        //     appearance: polylineApperance
        // })

        // viewer.scene.primitives.add(addGroundPolylineGeometry)
    },
    addKMLLayer(viewer, url, layerId, show) {
        const dataSource = Cesium.KmlDataSource.load(url, {
            camera: viewer.scene.camera,
            canvas: viewer.scene.canvas
        })
        dataSource.then(dataSource => {
            dataSource.name = layerId
            viewer.dataSources.add(dataSource);
            dataSource.show = show;
            // 获取实体数组
            var entities = dataSource.entities.values;
            for (var i = 0; i < entities.length; i++) {
                var entity = entities[i];

                if (Cesium.defined(entity.polygon)) {
                    // Cesium 将 GeoJson 的属性数据存储在实体的properties 属性中
                    entity.name = entity.properties.Text;
                    gridData.push({
                        label: entity.properties.Text._value,
                        id: entity.properties.ORIG_FID._value,
                    })
                    // 使用随机颜色样式作为填充材料.
                    entity.polygon.material = Cesium.Color.fromRandom({
                        red: 0.1,
                        maximumGreen: 0.5,
                        minimumBlue: 0.5,
                        alpha: 0.01
                    });

                    // Tells the polygon to color the terrain.
                    //ClassificationType.CESIUM_3D_TILE will color the 3D tileset, and ClassificationType.BOTH will color both the 3d tiles and terrain (BOTH is the default)
                    entity.polygon.classificationType = Cesium.ClassificationType.TERRAIN;

                    // 获取面的位置坐标信息
                    var polyPositions = entity.polygon.hierarchy.getValue(Cesium.JulianDate.now()).positions;
                    var polyCenter = Cesium.BoundingSphere.fromPoints(polyPositions).center; //获取面的中心点
                    polyCenter = Cesium.Ellipsoid.WGS84.scaleToGeodeticSurface(polyCenter);
                    polyCenter.z += 100
                    entity.position = polyCenter;
                    // Generate labels
                    entity.label = {
                        text: entity.name,
                        showBackground: true,//指定标签背景的可见性。
                        scale: 0.6,//原图像的比例
                        horizontalOrigin: Cesium.HorizontalOrigin.CENTER,//水平对齐方式
                        verticalOrigin: Cesium.VerticalOrigin.BOTTOM,//垂直对齐方式
                        distanceDisplayCondition: new Cesium.DistanceDisplayCondition(10.0, 80000.0),//设置于 Camera 距离的显示范围
                        disableDepthTestDistance: 100.0
                    };
                }
            }
        })
    },
    addXYZSLayer: (viewer, url, id, show) => {
        if (id == 'amap-vector') {
            // 创建一个新的 WebMercatorTilingScheme 并设置偏移
            const rectangleSouthwestInMeters = new Cesium.Cartesian2(-20037508.342789244 + 1000, -20037508.342789244 + 1000);
            const rectangleNortheastInMeters = new Cesium.Cartesian2(20037508.342789244 + 1000, 20037508.342789244 + 1000);

            const tilingScheme = new Cesium.WebMercatorTilingScheme({
                // rectangleSouthwestInMeters: rectangleSouthwestInMeters,
                // rectangleNortheastInMeters: rectangleNortheastInMeters
            });
            // 创建 UrlTemplateImageryProvider 并使用自定义的 tilingScheme
            var gaode = viewer.imageryLayers.addImageryProvider(new AMapImageryProvider({
                url: url,
                crs: 'WGS84',
                maximumLevel: 16,//没有设置会导致瓦片丢失出现黑色空白的情况
            }));
            gaode.id = id;
            gaode.show = show;
        } else {
            var gaode = viewer.imageryLayers.addImageryProvider(new Cesium.UrlTemplateImageryProvider({
                url: url,
                fileExtension: "png",
                maximumLevel: 19,//没有设置会导致瓦片丢失出现黑色空白的情况
                //   rectangle : Cesium.Rectangle.fromDegrees(113.757446187, 23.213844015,113.855416592, 23.317278851) 
            }));
            gaode.id = id;
            gaode.show = show;
        }

    },
    addGeoJSONData: (viewer, data, layerId, show) => {
        let jsonData = data;
        if (typeof data == 'string') {
            jsonData = JSON.parse(data)
        }
        // const dataSource = Cesium.GeoJsonDataSource.load(jsonData, {
        //     fill: Cesium.Color.BLUE.withAlpha(0.1), // 设置面的填充颜色
        //     stroke: Cesium.Color.RED, // 设置边界的颜色
        //     strokeWidth: 3, // 设置边界的宽度
        //     clampToGround:true,
        // });


        // return dataSource.then(function(dataSource) {
        //     if(jsonData.features[0].geometry.type.includes('Polygon')){
        //         var polygonEntities = dataSource.entities.values;
        //         for (var i = 0; i < polygonEntities.length; i++) {
        //             var entity = polygonEntities[i];
        //             entity.polygon.outline= true;
        //             entity.polygon.outlineColor=Cesium.Color.RED;  // 设置轮廓颜色为蓝色
        //             entity.polygon.outlineWidth= 3
        //             entity.polygon.height = 0
        //             entity.polygon.perPositionHeight = false;
        //             entity.polygon.heightReference = Cesium.HeightReference.CLAMP_TO_GROUND
        //         }
        //     }
        //     dataSource.name = layerId
        //     viewer.dataSources.add(dataSource);
        //     // viewer.zoomTo(dataSource);
        //     dataSource.show = show;
        //     viewer.flyTo(dataSource,{
        //         // maximumHeight:1000,
        //         offset:new Cesium.HeadingPitchRange(0, -90, 6000)
        //     });

        //     // // 获取实体数组
        //     // var entities = dataSource.entities.values;
        //     // for (var i = 0; i < entities.length; i++) {
        //     //     var entity = entities[i];

        //     //     if (Cesium.defined(entity.polygon)) {
        //     //         // Cesium 将 GeoJson 的属性数据存储在实体的properties 属性中
        //     //         entity.name = entity.properties.Text;
        //     //         gridData.push({
        //     //             label: entity.properties.Text._value,
        //     //             id: entity.properties.ORIG_FID._value,
        //     //         })
        //     //         // 使用随机颜色样式作为填充材料.
        //     //         entity.polygon.material = Cesium.Color.fromRandom({
        //     //             red : 0.1,
        //     //             maximumGreen : 0.5,
        //     //             minimumBlue : 0.5,
        //     //             alpha : 0.01
        //     //         });

        //     //         // Tells the polygon to color the terrain.
        //     //         //ClassificationType.CESIUM_3D_TILE will color the 3D tileset, and ClassificationType.BOTH will color both the 3d tiles and terrain (BOTH is the default)
        //     //         entity.polygon.classificationType = Cesium.ClassificationType.TERRAIN;

        //     //         // 获取面的位置坐标信息
        //     //         var polyPositions = entity.polygon.hierarchy.getValue(Cesium.JulianDate.now()).positions;
        //     //         var polyCenter = Cesium.BoundingSphere.fromPoints(polyPositions).center; //获取面的中心点
        //     //         polyCenter = Cesium.Ellipsoid.WGS84.scaleToGeodeticSurface(polyCenter);
        //     //         polyCenter.z += 100 
        //     //         entity.position = polyCenter;
        //     //         // Generate labels
        //     //         entity.label = {
        //     //             text : entity.name,
        //     //             showBackground : true,//指定标签背景的可见性。
        //     //             scale : 0.6,//原图像的比例
        //     //             horizontalOrigin : Cesium.HorizontalOrigin.CENTER,//水平对齐方式
        //     //             verticalOrigin : Cesium.VerticalOrigin.BOTTOM,//垂直对齐方式
        //     //             distanceDisplayCondition : new Cesium.DistanceDisplayCondition(10.0, 80000.0),//设置于 Camera 距离的显示范围
        //     //             disableDepthTestDistance : 100.0
        //     //         };
        //     //     }
        //     // }
        // });
        const tmpPrimitives = new Cesium.PrimitiveCollection();
        tmpPrimitives.name = layerId
        jsonData.features.forEach(feature => {
            let positions = [], polylinePositions = [];
            switch (feature.geometry.type) {
                case "LineString": {
                    positions = feature.geometry.coordinates.flat(2)
                    if (feature.geometry.coordinates[0].length == 2) {
                        polylinePositions = Cesium.Cartesian3.fromDegreesArray(positions)
                    } else if (feature.geometry.coordinates[0].length == 3) {
                        polylinePositions = Cesium.Cartesian3.fromDegreesArrayHeights(positions)
                    }
                    break;
                }
                case "MultiLineString": {
                    positions = feature.geometry.coordinates[0].flat(2)
                    if (feature.geometry.coordinates[0][0].length == 2) {
                        polylinePositions = Cesium.Cartesian3.fromDegreesArray(positions)
                    } else if (feature.geometry.coordinates[0][0].length == 3) {
                        polylinePositions = Cesium.Cartesian3.fromDegreesArrayHeights(positions)
                    }
                    break;
                }
                case "Polygon": {
                    // 不考虑有洞的情况
                    positions = feature.geometry.coordinates[0].flat(2)
                    if (feature.geometry.coordinates[0][0].length == 2) {
                        polylinePositions = Cesium.Cartesian3.fromDegreesArray(positions)
                    } else if (feature.geometry.coordinates[0][0].length == 3) {
                        polylinePositions = Cesium.Cartesian3.fromDegreesArrayHeights(positions)
                    }
                    break;
                }
                case "MultiPolygon": {
                    // 不考虑有洞的情况
                    positions = feature.geometry.coordinates[0].flat(2)
                    if (feature.geometry.coordinates[0][0][0].length == 2) {
                        polylinePositions = Cesium.Cartesian3.fromDegreesArray(positions)
                    } else if (feature.geometry.coordinates[0][0][0].length == 3) {
                        polylinePositions = Cesium.Cartesian3.fromDegreesArrayHeights(positions)
                    }
                    break;
                }
            }

            const instance = new Cesium.GeometryInstance({
                geometry: new Cesium.GroundPolylineGeometry({
                    positions: polylinePositions,
                    width: 1.0
                }),
                id: JSON.stringify(feature.properties)
            });
            tmpPrimitives.add(new Cesium.GroundPolylinePrimitive({
                geometryInstances: instance,
                appearance: new Cesium.PolylineMaterialAppearance({
                    material: Cesium.Material.fromType('Color', {
                        color: Cesium.Color.RED
                    })
                })
            }));
        })
        tmpPrimitives.show = show;
        viewer.scene.primitives.add(tmpPrimitives);

        // 获取图元的包围盒
        if (tmpPrimitives.length) {
            var polyPositions = tmpPrimitives.get(0).geometryInstances.geometry._positions;
            var boundingSphere = Cesium.BoundingSphere.fromPoints(polyPositions); //获取面的中心点
            if (boundingSphere) {
                // polyCenter.z+=2000
                // viewer.camera.flyTo({
                //     destination : polyCenter,
                //     orientation: {//使用欧拉角
                //         heading: Cesium.Math.toRadians(0.0),//使用弧度的形式
                //         pitch: Cesium.Math.toRadians(-90.0),
                //         roll: 0.0
                //     },
                //     duration:3,//设置飞行时间，单位为秒
                //     maximumHeight:1000,//最大的飞行高度
                // });
                viewer.camera.flyToBoundingSphere(boundingSphere);
            }

        }

    },
    loadTileset: async (viewer, tilesetUrl, name, isZoom = true) => {
        try {
            const tileset = await Cesium.Cesium3DTileset.fromUrl(tilesetUrl, {
                baseScreenSpaceError: 32,//默认16，定义了瓦片加载的基础屏幕空间误差。数值越大，加载的瓦片精度越低、越模糊，但加载速度会更快，可根据场景和需求适当增大该值，如设置为 1024 等
                skipScreenSpaceErrorFactor: 64,//控制跳过的屏幕空间误差级别，值越大，跳过的级别越多，加载速度越快，但可能会导致细节丢失，可根据实际情况调整，如设置为 16 等
                maximumScreenSpaceError: 120,//设置瓦片加载的最大屏幕空间误差，超过该值的瓦片将不会被加载。增大此值可以减少加载的数据量，加快加载速度，但可能会降低远处瓦片的显示质量，可将其数值加大
                // maximumMemoryUsage: 128, // 内存分配变小有利于倾斜摄影数据回收，提升性能体验
                // maximumNumberOfLoadedTiles: 300, // 默认值是200，根据需要调整(控制同时加载的最大tile数量) Cesium 1.80 版本开始，这个参数被弃用了

                skipLevels: 1,
                loadSiblings: false,
                progressiveResolutionHeightFraction: 0.5, // 数值偏于0能够让初始加载变得模糊
                cullWithChildrenBounds: true,//设置为 true，则当父tile不在视锥内时，不会加载子tile
                dynamicScreenSpaceError: true,
                dynamicScreenSpaceErrorDensity: 0.2,//数值加大，能让周边加载变快，但可能会影响一定的显示质量，可根据场景需求调整
                dynamicScreenSpaceErrorFactor: 24.0,
                dynamicScreenSpaceErrorHeightFalloff: 0.25,

                skipLevelOfDetail: true,
                immediatelyLoadDesiredLevelOfDetail: true
            });
            tileset.name = name;
            //  tileset.tileUnload.addEventListener(function(tile) {
            // console.log('A tile was loaded.',tile);
            // });
            //  console.log("tileset",tileset)
            // Adjust a tileset's height from the globe's surface.
            // const heightOffset = 200.0;
            // const boundingSphere = tileset.boundingSphere;
            // const cartographic = Cesium.Cartographic.fromCartesian(boundingSphere.center);
            // const surface = Cesium.Cartesian3.fromRadians(cartographic.longitude, cartographic.latitude, 0.0);
            // const offset = Cesium.Cartesian3.fromRadians(cartographic.longitude, cartographic.latitude, heightOffset);
            // const translation = Cesium.Cartesian3.subtract(offset, surface, new Cesium.Cartesian3());
            // tileset.modelMatrix = Cesium.Matrix4.fromTranslation(translation);
            viewer.scene.primitives.add(tileset);

            isZoom && viewer.zoomTo(tileset);
            return tileset;
        } catch (err) {
            // console.log("加载失败：",err)
            ElMessage.error("暂无地块【" + name + "】模型瓦片资源！")
        }
        return null;
    },
    addPoiLayer: async (viewer, url, id, show) => {
        // 获取标注数据源
        // const dataSource = new Cesium.CustomDataSource(id);
        // viewer.dataSources.add(dataSource);

        // 自定义poi图标的
        // await fetch(url).then(res=>res.json()).then((result)=>{
        //     result.features.forEach(feature=>{
        //         const color = Cesium.Color.SNOW;
        //         const iconurl = "/datas/local.png";
        //         const name = feature.properties.Name;
        //         const lon =  feature.geometry.coordinates[0];
        //         const lat =  feature.geometry.coordinates[1];
        //         const height =  feature.geometry.coordinates[2];
        //         dataLoading.poiIconLabelAdd(dataSource,lon, lat, height, name, color, iconurl);
        //     })
        // })

        // 方法一：使用 Cesium 自带的 Cluster 功能
        // 添加聚合功能的POI加载
        new Cesium.GeoJsonDataSource().load(url).then(async function (dataSource) {
            // 设置数据源属性
            viewer.dataSources.add(dataSource);
            dataSource.name = id;
            dataSource.show = show;

            // 设置聚合参数
            dataSource.clustering.enabled = true;
            dataSource.clustering.pixelRange = 50;
            dataSource.clustering.minimumClusterSize = 5;
            dataSource.clustering.clusterPoints = true;

            // foreach用于调用数组的每个元素，并将元素传递给回调函数。
            for (let entity of dataSource.entities.values) {
                const height = await getHeightByType(viewer, [entity.position._value], 'terrain');
                // 将点拉伸一定高度，防止被地形压盖

                entity.position = Cesium.Cartesian3.fromDegrees(entity.properties["经度"]._value, entity.properties["纬度"]._value, height[0]);
                // 使用大小为64*64的icon，缩小展示poi
                entity.billboard = {
                    image: '/datas/local.png',
                    width: 32,
                    height: 32,
                };
                entity.label = {
                    text: entity.properties["标准名称"],
                    font: 'bold 16px Microsoft YaHei',
                    // 竖直对齐方式
                    verticalOrigin: Cesium.VerticalOrigin.CENTER,
                    // 水平对齐方式
                    horizontalOrigin: Cesium.HorizontalOrigin.LEFT,
                    // 偏移量
                    pixelOffset: new Cesium.Cartesian2(15, 0),
                    fillColor: Cesium.Color.WHITE,//#409eff
                    backgroundColor: Cesium.Color.BLUE.withAlpha(0.7), // 设置背景颜色及透明度Cesium.Color.fromCssColorString("#409eff")
                    backgroundPadding: new Cesium.Cartesian2(10, 5), // 设置背景的内边距
                    showBackground: true // 显示背景
                }
            }
            // dataSource.entities.values.forEach(entity => {
            //     // 将点拉伸一定高度，防止被地形压盖
            //     // entity.position._value.z += 200.0;
            //     // 使用大小为64*64的icon，缩小展示poi
            //     entity.billboard = {
            //         image: '/datas/local.png',
            //         width: 32,
            //         height: 32,
            //     };
            //     entity.label = {
            //         text: entity.properties["标准名称"],
            //         font: 'bold 16px Microsoft YaHei',
            //         // 竖直对齐方式
            //         verticalOrigin: Cesium.VerticalOrigin.CENTER,
            //         // 水平对齐方式
            //         horizontalOrigin: Cesium.HorizontalOrigin.LEFT,
            //         // 偏移量
            //         pixelOffset: new Cesium.Cartesian2(15, 0),
            //         fillColor: Cesium.Color.WHITE,//#409eff
            //         backgroundColor: Cesium.Color.BLUE.withAlpha(0.7), // 设置背景颜色及透明度Cesium.Color.fromCssColorString("#409eff")
            //         backgroundPadding: new Cesium.Cartesian2(10, 5), // 设置背景的内边距
            //         showBackground: true // 显示背景
            //     }
            // });
            // 添加监听函数
            dataSource.clustering.clusterEvent.addEventListener(
                async function (clusteredEntities, cluster) {
                    try {
                        let time = new Date().getTime();
                        // 关闭自带的显示聚合数量的标签
                        cluster.label.show = false;
                        cluster.billboard.show = true;
                        cluster.billboard.verticalOrigin = Cesium.VerticalOrigin.BOTTOM;
                        // 根据聚合数量的多少设置不同层级的图片以及大小
                        if (clusteredEntities.length >= 6 && clusteredEntities.length < 12) {
                            cluster.billboard.width = 48;
                            cluster.billboard.height = 48;
                            cluster.billboard.image = dataLoading.combineIconAndLabel('/datas/cluster/1.png', clusteredEntities.length, 64);
                            // cluster.billboard.image = '/datas/cluster/1.png'
                        } else if (clusteredEntities.length >= 12 && clusteredEntities.length < 20) {
                            cluster.billboard.width = 56;
                            cluster.billboard.height = 56;
                            cluster.billboard.image = dataLoading.combineIconAndLabel('/datas/cluster/2.png', clusteredEntities.length, 64);
                            // cluster.billboard.image = '/datas/cluster/2.png'
                        } else if (clusteredEntities.length >= 20) {
                            cluster.billboard.width = 72;
                            cluster.billboard.height = 72;
                            cluster.billboard.image = dataLoading.combineIconAndLabel('/datas/cluster/3.png', clusteredEntities.length, 64);
                            // cluster.billboard.image = '/datas/cluster/3.png'
                        }
                    } catch (err) {
                        console.log(err)
                    }
                }
            )
        });

        // dataSource.show = show;

        // 方法二：手动实现简单聚合算法
        // // 初始化标签实例
        // const billboardsCollectionCombine = new Cesium.BillboardCollection();
        // // 初始化实体
        // const primitives = viewer.scene.primitives.add(
        //     new Cesium.PrimitiveCollection()
        // );
        // fetch(url).then(res=>res.json()).then((result)=>{
        //     console.log(result);
        //     const { features } = result;
        //     // formatClusterPoint(features);
        //     var scene = viewer.scene;
        //     var primitivecluster = new PrimitiveCluster();

        //     //与entitycluster相同设置其是否聚合 以及最大最小值
        //     primitivecluster.enabled = true;
        //     primitivecluster.pixelRange = 60;
        //     primitivecluster.minimumClusterSize = 5;
        //     // primitivecluster._pointCollection = pointCollection;
        //     // primitivecluster._labelCollection = labelCollection;

        //     for (let i = 0; i < features.length; i++) {
        //         const feature = features[i];
        //         const coordinates = feature.geometry.coordinates;
        //         const position = Cesium.Cartesian3.fromDegrees(
        //             coordinates[0],
        //             coordinates[1]
        //         );

        //         // 带图片的点
        //         billboardsCollectionCombine.add({
        //             image:"/datas/local.png",
        //             width: 32,
        //             height: 32,
        //             position,
        //         });
        //     }
        //     // 将数据传给primitivecluster的标签属性
        //     primitivecluster._billboardCollection = billboardsCollectionCombine;
        //     // 初始化
        //     primitivecluster._initialize(scene);
        //     // 将标签数据添加到实体中
        //     primitives.add(primitivecluster);

        //     // 监听相机缩放
        //     primitivecluster.clusterEvent.addEventListener( (clusteredEntities, cluster) => {
        //             // 关闭自带的显示聚合数量的标签
        //             cluster.label.show = false;
        //             cluster.billboard.show = true;
        //             cluster.billboard.verticalOrigin = Cesium.VerticalOrigin.BOTTOM;

        //             // 根据聚合数量的多少设置不同层级的图片以及大小
        //             cluster.billboard.image = dataLoading.combineIconAndLabel(
        //                 "/datas/cluster/3.png",
        //                 clusteredEntities.length,
        //                 64
        //             );
        //             // cluster.billboard.image = "/datas/cluster/3.png";
        //             cluster.billboard._imageHeight = 60;
        //             cluster.billboard._imageWidth = 60;
        //             cluster.billboard._dirty = false;
        //             cluster.billboard.width = 40;
        //             cluster.billboard.height = 40;
        //     });
        //     return primitivecluster;
        // })

        // // 设置数据源属性
        // const clusterSource = new Cesium.CustomDataSource(id);
        // clusterSource.show = show;
        // viewer.dataSources.add(clusterSource);
        // // 定义聚合阈值（单位：像素）
        // const pixelThreshold = 60;
        // // 存储聚合后的点
        // let clusters = [];
        // fetch(url).then(res => res.json()).then(function (featureCollection) {
        //     const points = [];
        //     featureCollection.features.forEach(feature => {
        //         points.push({ 
        //             position: Cesium.Cartesian3.fromDegrees(feature.geometry.coordinates[0], feature.geometry.coordinates[1], 200),
        //             name: feature.properties["Name"]
        //         })
        //     })

        //     // 将像素阈值转换为地理距离阈值的函数
        //     function pixelToDistanceThreshold() {
        //         const scene = viewer.scene;
        //         const camera = viewer.camera;
        //         const frustum = camera.frustum;
        //         const width = scene.canvas.clientWidth;
        //         const height = scene.canvas.clientHeight;
        //         const center = new Cesium.Cartesian2(width / 2, height / 2);
        //         const centerRay = camera.getPickRay(center);
        //         const centerPosition = scene.globe.pick(centerRay, scene);
        //         if (!centerPosition) return 0;

        //         const offset = new Cesium.Cartesian2(center.x + pixelThreshold, center.y);
        //         const offsetRay = camera.getPickRay(offset);
        //         const offsetPosition = scene.globe.pick(offsetRay, scene);
        //         if (!offsetPosition) return 0;

        //         return Cesium.Cartesian3.distance(centerPosition, offsetPosition);
        //     }

        //     // 进行点聚合的函数
        //     function performClustering() {
        //         const distanceThreshold = pixelToDistanceThreshold();
        //         clusters = [];
        //         for (let i = 0; i < points.length; i++) {
        //             let foundCluster = false;
        //             for (let j = 0; j < clusters.length; j++) {
        //                 const distance = Cesium.Cartesian3.distance(points[i].position, clusters[j].center);
        //                 if (distance < distanceThreshold) {
        //                     // 将点添加到已有的聚合点中
        //                     clusters[j].points.push(points[i]);
        //                     // 重新计算聚合点的中心
        //                     let sumX = 0;
        //                     let sumY = 0;
        //                     let sumZ = 0;
        //                     for (let k = 0; k < clusters[j].points.length; k++) {
        //                         sumX += clusters[j].points[k].position.x;
        //                         sumY += clusters[j].points[k].position.y;
        //                         sumZ += clusters[j].points[k].position.z;
        //                     }
        //                     clusters[j].center = new Cesium.Cartesian3(
        //                         sumX / clusters[j].points.length,
        //                         sumY / clusters[j].points.length,
        //                         sumZ / clusters[j].points.length
        //                     );
        //                     clusters[j].name = clusters[j].points.length;
        //                     clusters[j].isCluster = true
        //                     foundCluster = true;
        //                     break;
        //                 }
        //             }
        //             if (!foundCluster) {
        //                 // 创建一个新的聚合点
        //                 clusters.push({
        //                     center: points[i].position,
        //                     points: [points[i]],
        //                     name: points[i].name,
        //                     isCluster:false
        //                 });
        //             }
        //         }
        //     }

        //     // 在Cesium中显示聚合点的函数
        //     function displayClusters() {
        //         const dataSource = viewer.dataSources.getByName(id)[0]
        //         // 先移除之前的实体
        //         dataSource.entities.removeAll();
        //         for (let i = 0; i < clusters.length; i++) {
        //             let billboard= {
        //                 image: '/datas/local.png',
        //                 width: 32,
        //                 height: 32,
        //                 heightReference: Cesium.HeightReference.CLAMP_TO_GROUND 
        //             }

        //             let label = {
        //                 text: clusters[i].name+"",
        //                 font: 'bold 15px Microsoft YaHei',
        //                 // 竖直对齐方式
        //                 verticalOrigin: Cesium.VerticalOrigin.CENTER,
        //                 // 水平对齐方式
        //                 horizontalOrigin: Cesium.HorizontalOrigin.LEFT,
        //                 // 偏移量
        //                 pixelOffset: new Cesium.Cartesian2(15, 0),
        //             }
        //             // if(clusters[i].isCluster){
        //             //     const num = clusters[i].points.length
        //             //     if (num >= 6 && num < 12) {
        //             //         billboard.width = 48;
        //             //         billboard.height = 48;
        //             //         // billboard.image = dataLoading.combineIconAndLabel('/datas/cluster/1.png', num, 64);
        //             //         billboard.image = '/datas/cluster/1.png'
        //             //     } else if (num >= 12 && num < 20) {
        //             //         billboard.width = 56;
        //             //         billboard.height = 56;
        //             //         // billboard.image = dataLoading.combineIconAndLabel('/datas/cluster/2.png', num, 64);
        //             //         billboard.image = '/datas/cluster/1.png'
        //             //     } else if (num >= 20) {
        //             //         billboard.width = 72;
        //             //         billboard.height = 72;
        //             //         // billboard.image = dataLoading.combineIconAndLabel('/datas/cluster/3.png', num, 64);
        //             //         billboard.image = '/datas/cluster/1.png'
        //             //     }
        //             // }
        //             dataSource.entities.add({
        //                 position: clusters[i].center,
        //                 point: {
        //                     color: Cesium.Color.RED,
        //                     pixelSize: clusters[i].points.length * 2 // 聚合点的大小根据包含的点数而定
        //                 },
        //                 "label": label,
        //                 "billboard":billboard
        //             });
        //         }
        //     }

        //     // 初始执行点聚合和显示
        //     performClustering();
        //     displayClusters();

        //     // 监听相机变化事件
        //     viewer.camera.changed.addEventListener(() => {
        //         performClustering();
        //         displayClusters();
        //     });
        // })

    },
    /**
     * @description: 将图片和文字合成新图标使用（参考Cesium源码）
     * @param {*} url：图片地址
     * @param {*} label：文字
     * @param {*} size：画布大小
     * @return {*} 返回canvas
     */
    combineIconAndLabel: (url, label, size) => {
        // 创建画布对象
        let canvas = document.createElement('canvas');
        canvas.width = size;
        canvas.height = size;
        let ctx = canvas.getContext("2d");

        let promise = new Cesium.Resource.fetchImage(url).then(image => {
            // 异常判断
            try {
                ctx.drawImage(image, 0, 0);
            } catch (e) {
                console.log(e);
            }

            // 渲染字体
            // font属性设置顺序：font-style, font-variant, font-weight, font-size, line-height, font-family
            ctx.fillStyle = Cesium.Color.WHITE.toCssColorString();
            ctx.font = 'bold 20px Microsoft YaHei';
            ctx.textAlign = "center";
            ctx.textBaseline = "middle";
            ctx.fillText(label, size / 2, size / 2);

            return canvas;
        });
        return promise;
    },
    /**
  * 用于添加poi的icon和label的函数
  * @param {*} lon ：经度
  * @param {*} lat ：纬度
  * @param {*} name ：标签内容
  * @param {*} color ：底部圆和横线的颜色
  * @param {*} url ：icon地址
  */
    poiIconLabelAdd: (labelDataSource, lon, lat, height, name, color, url) => {


        //添加图标和标注
        labelDataSource.entities.add({
            name: name,
            position: Cesium.Cartesian3.fromDegrees(lon, lat, height + 30),
            // 图标
            billboard: {
                image: url,
                width: 50,
                height: 50,
            },
            label: {
                //文字标签
                text: name,
                font: "20px sans-serif",
                style: Cesium.LabelStyle.FILL,
                // 对齐方式(水平和竖直)
                horizontalOrigin: Cesium.HorizontalOrigin.LEFT,
                verticalOrigin: Cesium.VerticalOrigin.CENTER,
                pixelOffset: new Cesium.Cartesian2(20, -2),
                showBackground: true,
                backgroundColor: new Cesium.Color.fromBytes(0, 70, 24),
            },
        });

        // // 先画线后画点，防止线压盖点
        // let linePositions = [];
        // linePositions.push(new Cesium.Cartesian3.fromDegrees(lon, lat, height));
        // linePositions.push(
        //     new Cesium.Cartesian3.fromDegrees(lon, lat, height + 300)
        // );
        // labelDataSource.entities.add({
        //     polyline: {
        //         positions: linePositions,
        //         width: 1.5,
        //         material: color,
        //     },
        // })
        // // 画点
        // labelDataSource.entities.add({
        //     // 给初始点位设置一定的离地高度，否者会被压盖
        //     position: Cesium.Cartesian3.fromDegrees(lon, lat, height),
        //     point: {
        //     color: color,
        //     pixelSize: 15,
        //     },
        // });
    }

}


export default dataLoading;